@@ -16,7 +16,7 @@
 #include "tools.h"
 #include "report.h"
 
-static int _vgs_single(struct cmd_context *cmd __attribute((unused)),
+static int _vgs_single(struct cmd_context *cmd __attribute__((unused)),
 		       const char *vg_name, struct volume_group *vg,
 		       void *handle)
 {
@@ -33,9 +33,6 @@ static int _vgs_single(struct cmd_context *cmd __attribute((unused)),
 static int _lvs_single(struct cmd_context *cmd, struct logical_volume *lv,
 		       void *handle)
 {
-	if (!arg_count(cmd, all_ARG) && !lv_is_visible(lv))
-		return ECMD_PROCESSED;
-
 	if (!report_object(handle, lv->vg, lv, NULL, NULL, NULL)) {
 		stack;
 		return ECMD_FAILED;
@@ -44,7 +41,7 @@ static int _lvs_single(struct cmd_context *cmd, struct logical_volume *lv,
 	return ECMD_PROCESSED;
 }
 
-static int _segs_single(struct cmd_context *cmd __attribute((unused)),
+static int _segs_single(struct cmd_context *cmd __attribute__((unused)),
 			struct lv_segment *seg, void *handle)
 {
 	if (!report_object(handle, seg->lv->vg, seg->lv, NULL, seg, NULL)) {
@@ -54,7 +51,6 @@ static int _segs_single(struct cmd_context *cmd __attribute((unused)),
 
 	return ECMD_PROCESSED;
 }
-
 static int _pvsegs_sub_single(struct cmd_context *cmd,
 			      struct volume_group *vg,
 			      struct pv_segment *pvseg, void *handle)
@@ -64,35 +60,36 @@ static int _pvsegs_sub_single(struct cmd_context *cmd,
 
 	struct volume_group _free_vg = {
 		.cmd = cmd,
-		.name = (char *)"",
+		.name = "",
+		.vgmem = NULL,
 	};
 
 	struct logical_volume _free_logical_volume = {
 		.vg = vg ?: &_free_vg,
-		.name = (char *) "",
-	        .snapshot = NULL,
+		.name = "",
+		.snapshot = NULL,
 		.status = VISIBLE_LV,
 		.major = -1,
 		.minor = -1,
 	};
 
 	struct lv_segment _free_lv_segment = {
-        	.lv = &_free_logical_volume,
-        	.le = 0,
-        	.status = 0,
-        	.stripe_size = 0,
-        	.area_count = 0,
-        	.area_len = 0,
-        	.origin = NULL,
-        	.cow = NULL,
-        	.chunk_size = 0,
-        	.region_size = 0,
-        	.extents_copied = 0,
-        	.log_lv = NULL,
-        	.areas = NULL,
+		.lv = &_free_logical_volume,
+		.le = 0,
+		.status = 0,
+		.stripe_size = 0,
+		.area_count = 0,
+		.area_len = 0,
+		.origin = NULL,
+		.cow = NULL,
+		.chunk_size = 0,
+		.region_size = 0,
+		.extents_copied = 0,
+		.log_lv = NULL,
+		.areas = NULL,
 	};
 
-        _free_lv_segment.segtype = get_segtype_from_string(cmd, "free");
+	_free_lv_segment.segtype = get_segtype_from_string(cmd, "free");
 	_free_lv_segment.len = pvseg->len;
 	dm_list_init(&_free_vg.pvs);
 	dm_list_init(&_free_vg.lvs);
@@ -106,10 +103,11 @@ static int _pvsegs_sub_single(struct cmd_context *cmd,
 
 	if (!report_object(handle, vg, seg ? seg->lv : &_free_logical_volume, pvseg->pv,
 			   seg ? : &_free_lv_segment, pvseg)) {
-		stack;
 		ret = ECMD_FAILED;
+                goto_out;
 	}
 
+ out:
 	return ret;
 }
 
@@ -136,6 +134,7 @@ static int _pvs_single(struct cmd_context *cmd, struct volume_group *vg,
 	int ret = ECMD_PROCESSED;
 	const char *vg_name = NULL;
 	struct volume_group *old_vg = vg;
+	char uuid[64] __attribute__((aligned(8)));
 
 	if (is_pv(pv) && !is_orphan(pv) && !vg) {
 		vg_name = pv_vg_name(pv);
@@ -143,21 +142,34 @@ static int _pvs_single(struct cmd_context *cmd, struct volume_group *vg,
 		vg = vg_read(cmd, vg_name, (char *)&pv->vgid, 0);
 		if (vg_read_error(vg)) {
 			log_error("Skipping volume group %s", vg_name);
+			release_vg(vg);
 			return ECMD_FAILED;
 		}
 
 		/*
 		 * Replace possibly incomplete PV structure with new one
-		 * allocated in vg_read_internal() path.
+		 * allocated in vg_read.
 		*/
-		if (!(pvl = find_pv_in_vg(vg, pv_dev_name(pv)))) {
-			log_error("Unable to find \"%s\" in volume group \"%s\"",
-				  pv_dev_name(pv), vg->name);
+		if (!is_missing_pv(pv)) {
+			if (!(pvl = find_pv_in_vg(vg, pv_dev_name(pv)))) {
+				log_error("Unable to find \"%s\" in volume group \"%s\"",
+					  pv_dev_name(pv), vg->name);
+				ret = ECMD_FAILED;
+				goto out;
+			}
+		} else if (!(pvl = find_pv_in_vg_by_uuid(vg, &pv->id))) {
+			if (!id_write_format(&pv->id, uuid, sizeof(uuid))) {
+				stack;
+				uuid[0] = '\0';
+			}
+
+			log_error("Unable to find missing PV %s in volume group %s",
+				  uuid, vg->name);
 			ret = ECMD_FAILED;
 			goto out;
 		}
 
-		 pv = pvl->pv;
+		pv = pvl->pv;
 	}
 
 	if (!report_object(handle, vg, NULL, pv, NULL, NULL)) {
@@ -170,7 +182,7 @@ out:
 		unlock_vg(cmd, vg_name);
 
 	if (!old_vg)
-		vg_release(vg);
+		release_vg(vg);
 
 	return ret;
 }
@@ -327,11 +339,9 @@ static int _report(struct cmd_context *cmd, int argc, char **argv,
 	}
 
 	/* -O overrides default sort settings */
-	if (arg_count(cmd, sort_ARG))
-		keys = arg_str_value(cmd, sort_ARG, "");
+	keys = arg_str_value(cmd, sort_ARG, keys);
 
-	if (arg_count(cmd, separator_ARG))
-		separator = arg_str_value(cmd, separator_ARG, " ");
+	separator = arg_str_value(cmd, separator_ARG, separator);
 	if (arg_count(cmd, separator_ARG))
 		aligned = 0;
 	if (arg_count(cmd, aligned_ARG))
@@ -353,6 +363,8 @@ static int _report(struct cmd_context *cmd, int argc, char **argv,
 					  separator, aligned, buffered,
 					  headings, field_prefixes, quoted,
 					  columns_as_rows))) {
+		if (!strcasecmp(options, "help") || !strcmp(options, "?"))
+			return r;
 		stack;
 		return ECMD_FAILED;
 	}
